@@ -15,7 +15,7 @@
 """Extract message strings from python modules, page template files
 and ZCML files.
 
-$Id: extract.py 115261 2010-07-31 12:46:11Z icemac $
+$Id: extract.py 123777 2011-12-12 15:11:31Z mgedmin $
 """
 __docformat__ = 'restructuredtext'
 
@@ -96,6 +96,16 @@ class POTEntry(object):
     msgid "test"
     msgstr ""
     <BLANKLINE>
+
+    Unicode can be used in msgids and default values
+
+    >>> entry = POTEntry(Message(u"\u263B", default=u"\u253A"))
+    >>> entry.write(FakeFile())
+    #. Default: "\342\224\272"
+    msgid "\342\230\273"
+    msgstr ""
+    <BLANKLINE>
+
     """
 
     implements(IPOTEntry)
@@ -103,31 +113,40 @@ class POTEntry(object):
     def __init__(self, msgid, comments=None):
         self.msgid = msgid
         self.comments = comments or ''
+        self.locations = []
 
     def addComment(self, comment):
         self.comments += comment + '\n'
 
     def addLocationComment(self, filename, line):
-        self.comments += '#: %s:%s\n' % (
-            filename.replace(os.sep, '/'), line)
+        filename = filename.replace(os.sep, '/')
+        self.locations.append((filename, line))
+        self.locations.sort()
 
     def write(self, file):
         if self.comments:
             file.write(self.comments)
+        for filename, line in self.locations:
+            file.write('#: %s:%s\n' % (filename, line))
         if (isinstance(self.msgid, Message) and
             self.msgid.default is not None):
-            default = self.msgid.default.strip()
+            default = self.msgid.default.strip().encode(DEFAULT_CHARSET)
             lines = normalize(default).split("\n")
             lines[0] = "#. Default: %s\n" % lines[0]
             for i in range(1, len(lines)):
                 lines[i] = "#.  %s\n" % lines[i]
             file.write("".join(lines))
-        file.write('msgid %s\n' % normalize(self.msgid))
+        file.write('msgid %s\n' % normalize(self.msgid.encode(DEFAULT_CHARSET)))
         file.write('msgstr ""\n')
         file.write('\n')
 
     def __cmp__(self, other):
-        return cmp(self.comments, other.comments)
+        return cmp((self.locations, self.msgid),
+                   (other.locations, other.msgid))
+
+    def __repr__(self):
+        return '<POTEntry: %r>' % self.msgid
+
 
 class POTMaker(object):
     """This class inserts sets of strings into a POT file.
@@ -147,8 +166,8 @@ class POTMaker(object):
                 self.catalog[msgid] = POTEntry(msgid)
 
             for filename, lineno in locations:
-                if base_dir is not None:
-                    filename = filename.replace(base_dir, '')
+                if base_dir:
+                    filename = strip_base_dir(filename, base_dir)
                 self.catalog[msgid].addLocationComment(filename, lineno)
 
     def _getProductVersion(self):
@@ -599,6 +618,22 @@ def normalize_path(path):
         cwd = os.environ['PWD']
     return os.path.normpath(os.path.join(cwd, path))
 
+def strip_base_dir(filename, base_dir):
+    """Strip base directory from filename if it starts there.
+
+        >>> strip_base_dir('/path/to/base/relpath/to/file',
+        ...                '/path/to/base/')
+        'relpath/to/file'
+
+        >>> strip_base_dir('/path/to/somewhere/else/relpath/to/file',
+        ...                '/path/to/base/')
+        '/path/to/somewhere/else/relpath/to/file'
+
+    """
+    if filename.startswith(base_dir):
+        filename = filename[len(base_dir):]
+    return filename
+
 def main(argv=None):
     if argv is None:
         argv = sys.argv[1:]
